#ifndef __XLIC_CLIENT__
#define __XLIC_CLIENT__

#include <xtl/exception.hpp>
#include <xtl/unix_socket.hpp>
#include <xlic/types.hpp>

namespace xlic {

class client_exception : public xtl::exception {
public:
	inline client_exception(const char * file, int line, const char * reason)
	: exception(file, line, "%s", reason) {}
};

class client {
public:
	client(const char * filename);
	client(client && other) noexcept;
	client & operator=(client && other) noexcept;
	~client() noexcept;

	void fetch_licenses();
	void register_license(const std::string & key);
	void request_license(const std::string & name, const std::string & username, const std::string & password);
	void unregister_license(const std::string & key);
	xlic::license_list list_licenses();
	void update_licenses();
	void clear_licenses();
	xlic::system_name system_name();
	void set_system_name(const std::string & system_name);

private:
	xtl::unix_socket m_socket;
	xlic::request_id m_next_request_id;
};

} // namespace xlic

#endif
