#ifndef __XLIC_ROCKEY__
#define __XLIC_ROCKEY__

#include <cstddef>
#include <cstdint>
#include <vector>
#include <string>
#include <exception>

class rockey {
public:
	enum algo_mode { A, B, C };

	struct info {
		uint32_t version;
		uint32_t hwid;
		uint64_t timestamp;
		uint16_t magic;
		std::string serial;
	};

	inline rockey() noexcept
	: m_opened(false)
	, m_handle(0)
	, m_hwid(0)
	{}

	inline rockey(uint16_t handle, uint32_t hwid) noexcept
	: m_opened(false)
	, m_handle(handle)
	, m_hwid(hwid)
	{}

	rockey(const rockey & other) = delete;
	rockey & operator=(const rockey & other) = delete;

	inline rockey(rockey && other) noexcept
	: m_opened(false)
	, m_handle(0)
	, m_hwid(0)
	{
		swap(other);
	}

	inline rockey & operator=(rockey && other) noexcept {
		swap(other);
		return *this;
	}

	inline ~rockey() noexcept {
		if (m_opened) {
			close();
		}
	}

	inline void swap(rockey & other) noexcept {
		std::swap(m_opened, other.m_opened);
		std::swap(m_handle, other.m_handle);
		std::swap(m_hwid, other.m_hwid);
	}

	inline uint32_t hwid() const noexcept {
		return m_hwid;
	}

	// low-level functions
	static std::vector<rockey> enumerate(uint16_t p1, uint16_t p2, uint16_t p3 = 0, uint16_t p4 = 0);
	void open(uint16_t p1, uint16_t p2, uint16_t p3 = 0, uint16_t p4 = 0);
	void close() noexcept;
	void read_memory(void * buffer, uint16_t size, uint16_t offset = 0);
	void write_memory(const void * buffer, uint16_t size, uint16_t offset);
	void set_module(uint16_t index, uint16_t value);
	void set_algo(uint16_t index, const std::string & algorithm);
	void call_algo(uint16_t index, algo_mode mode, uint16_t & a, uint16_t & b, uint16_t & c, uint16_t & d, uint32_t arg = 0);

	// high-level functions
	info read();
	bool is_xorcom_dongle(const std::string & serial, uint16_t magic);
	void program(const std::string & serial);

private:
	bool m_opened;
	uint16_t m_handle;
	uint32_t m_hwid;
};

class rockey_exception : public std::exception {
public:
	inline rockey_exception(uint16_t code) noexcept
	: m_code(code)
	{}

	const char * what() const noexcept override;

private:
	uint16_t m_code;
};

#endif
