#include <xlic/json.hpp>

#include <xtl/util.hpp>
#include <xtl/json.hpp>

namespace xlic {

inline std::string escape(const std::string & input) {
	// Really dumb implementation, sue me
	std::string output;
	for (const char c : input) {
		switch (c) {
			case '"': { output += "\\\""; break; }
			case '\\': { output += "\\\\"; break; }
			case '\b': { output += "\\\b"; break; }
			case '\f': { output += "\\\f"; break; }
			case '\n': { output += "\\\n"; break; }
			case '\r': { output += "\\\r"; break; }
			case '\t': { output += "\\\t"; break; }
			default: { output += c; break; }
		}
	}
	return output;
}

inline std::string encode(const std::string & string) {
	return xtl::concat("\"", escape(string), "\"");
}

inline std::string encode(const xtl::mac_address & mac_address) {
	return encode(to_string(mac_address));
}

inline std::string encode(const xlic::license_key & license_key) {
	return encode(to_string(license_key));
}

inline std::string encode(const xtl::rsa_signature & signature) {
	return encode(to_string(signature));
}

inline std::string encode(const xlic::mac_address_list & mac_addresses) {
	std::string result("[");
	auto it = mac_addresses.begin();
	if (it != mac_addresses.end()) {
		result += encode(*it++);
	}
	while (it != mac_addresses.end()) {
		result += xtl::concat(",", encode(*it++));
	}
	result += "]";
	return result;
}

inline std::string encode(const xlic::license & license) {
	return xtl::concat(
		"{",
			"\"license_key\":", encode(license.license_key), ","
			"\"signature\":", encode(license.signature),
		"}"
	);
}

inline std::string encode(const xlic::license_list & licenses) {
	std::string result("[");
	auto it = licenses.begin();
	if (it != licenses.end()) {
		result += encode((it++)->second);
	}
	while (it != licenses.end()) {
		result += xtl::concat(",", encode((it++)->second));
	}
	result += "]";
	return result;
}

inline std::string encode(const xlic::extension_count extensions) {
	std::string result;
	const auto len = snprintf(nullptr, 0, "%d", extensions);
	if (len > 0) {
		result.resize(len);
		snprintf(&result[0], len + 1, "%d", extensions);
	}
	return result;
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses
) {
	return xtl::concat(
		"{"
			"\"serial_number\":", encode(serial_number), ","
			"\"mac_addresses\":", encode(mac_addresses), ""
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::system_name & system_name
) {
	return xtl::concat(
		"{"
			"\"serial_number\":", encode(serial_number), ","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"system_name\":", encode(system_name), ""
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::system_name & system_name,
	const xlic::license_key & license_key
) {
	return xtl::concat(
		"{"
			"\"serial_number\":", encode(serial_number), ","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"system_name\":", encode(system_name), ","
			"\"license_key\":", encode(license_key), ""
		"}"
	);
}

std::string json_encode(
	const xtl::rsa_signature & signature,
	const xlic::license_key & license_key
) {
	return xtl::concat(
		"{"
			"\"signature\":", encode(signature), ","
			"\"license_key\":", encode(license_key),
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::license_list & licenses,
	const xlic::system_name & system_name,
	const xlic::extension_count extensions
) {
	return xtl::concat(
		"{"
			"\"serial_number\":", encode(serial_number), ","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"licenses\":", encode(licenses), ","
			"\"system_name\":", encode(system_name), ","
			"\"extensions\":", encode(extensions),
		"}"
	);
}

std::string json_encode(
	const xlic::serial_number & serial_number,
	const xlic::mac_address_list & mac_addresses,
	const xlic::system_name & system_name,
	const xlic::license_name & name,
	const xlic::username & username,
	const xlic::password & password
) {
	return xtl::concat(
		"{"
			"\"serial_number\":", encode(serial_number), ","
			"\"mac_addresses\":", encode(mac_addresses), ","
			"\"system_name\":", encode(system_name), ","
			"\"name\":", encode(name), ","
			"\"username\":", encode(username), ","
			"\"password\":", encode(password),
		"}"
	);
}

} // namespace xlic
