#include <xlic/set_system_name.hpp>

#include <xtl/exception.hpp>
#include <xtl/https_stream.hpp>
#include <xlic/json.hpp>
#include <xlic/worker.hpp>
#include <xlic/types.hpp>

#if defined(DEBUG_UNREGISTER_LICENSE) || defined(DEBUG_ALL)
#define fdebug(fmt, ...) printf("set_system_name[%d]: " fmt "\n", __LINE__, __VA_ARGS__);
#define debug(msg) printf("set_system_name[%d]: " msg "\n", __LINE__);
#else
#define fdebug(fmt, ...)
#define debug(msg)
#endif

namespace xlic {

set_system_name::set_system_name(
	xlic::session * session,
	xlic::worker & worker,
	xlic::request_id request_id,
	const xlic::system_name & system_name
)
: task(session, worker, request_id)
, m_result_code(xlic::result_code::unknown)
, m_thread([this, system_name] () {
	try {
		if (m_worker.foreign_license_file()) {
			m_result_code = xlic::result_code::foreign;
		} else {
			xtl::https_stream stream(XLIC_SERVER_HOSTNAME);
			stream.post("/set_system_name");
			stream.useragent(XLIC_USERAGENT);
			stream.content_type(XLIC_CONTENT_TYPE);
			stream << xlic::json_encode(m_worker.serial_number(), m_worker.mac_addresses(), system_name);
			stream.flush();
			const xtl::json js(stream);
			if (js["message"].string() == "success") {
				m_worker.system_name(system_name);
				m_result_code = xlic::result_code::success;
			} else {
				m_result_code = xlic::result_code::rejected;
			}
		}
	} catch (const xtl::https_stream_exception &) {
		m_result_code = xlic::result_code::server_error;
	} catch (const xtl::json_exception &) {
		m_result_code = xlic::result_code::parse_error;
	} catch (const xtl::end_of_stream &) {
		m_result_code = xlic::result_code::server_error;
	} catch (const std::exception & e) {
		fdebug("%s\n", e.what());
		m_result_code = xlic::result_code::generic_error;
	}
	m_worker.completed(this);
}) {}

void set_system_name::write_result(xtl::ostream & stream) {
	if (m_result_code == xlic::result_code::success) {
		stream << xlic::packet_size(xlic::packet_header_size);
		stream << xlic::packet_type::success;
		stream << m_request_id;
	} else {
		stream << xlic::packet_size(xlic::packet_header_size + sizeof(m_result_code));
		stream << xlic::packet_type::error;
		stream << m_request_id;
		stream << m_result_code;
	}
}

} // namespace xlic
